import json
import os
from typing import List, Optional, Tuple, TYPE_CHECKING

import click

from ..models import MigrationPlan
from ..util import get_unusable_idp_domains
from .decorators import requires_logged_in, requires_migration_plan

if TYPE_CHECKING:
    from ..typing.gcs_documents import IdentityMappingDict  # noqa: F401


@click.command("custom", help="Configure a custom identity mapping")
@click.option(
    "--domain",
    "-d",
    required=True,
    multiple=True,
    metavar="DOMAIN",
    help="Auth Domain to allow access to the endpoint",
)
@click.option(
    "--identity-mapping",
    "-i",
    multiple=True,
    metavar="JSON_OR_FILE",
    help="Either a JSON object or the name of a file that contains a json object. "
    "This object is parsed as an identity mapping document and added to the "
    "endpoint's policies.",
)
@requires_logged_in
@requires_migration_plan
@click.pass_context
def identity_mapping_custom(
    ctx: click.Context,
    domain: str,
    identity_mapping: Optional[Tuple[str]],
    migration_plan: MigrationPlan,
) -> None:
    if not migration_plan.gsi_authz_configuration:
        raise click.ClickException("Migration plan has no GSI configuration")
    if not migration_plan.gsi_authz_configuration.callouts:
        raise click.ClickException("Migration plan has no callouts configured")
    if not migration_plan.storage_gateway:
        raise click.ClickException("Migration plan has no storage gateway")

    # Load identity mapping if specified on the command-line
    identity_mapping_json: Optional[List["IdentityMappingDict"]] = None
    if identity_mapping:
        identity_mapping_json = []
        for idm in identity_mapping:
            if os.path.exists(idm):
                with open(idm, "r") as f:
                    identity_mapping_json.append(json.load(f))
            else:
                identity_mapping_json.append(json.loads(idm))

    # Verify the domains with Auth before adding them to the gateway
    unusable_domains = get_unusable_idp_domains(set(domain))

    if unusable_domains:
        click.echo(
            "The following domains are not valid identity providers "
            f"recognized but Globus Auth: {', '.join(unusable_domains)}.",
            err=True,
        )
        ctx.exit(1)

    # Add the domains and identity mappings to the storage gateway
    migration_plan.storage_gateway.add_allowed_domains(
        domain, identity_mappings=identity_mapping_json
    )
    migration_plan.dump_storage_gateway()
    migration_plan.dump_gsi_authz_configuration()
